package org.crosswire.android.bishop;

import java.util.Vector;

import org.crosswire.android.sword.SWMgr;
import org.crosswire.android.sword.SWModule;

import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.ViewGroup.LayoutParams;
import android.widget.LinearLayout;
import android.widget.TextView;

public class GotoDialog extends Dialog {

	LinearLayout mainLayout;
	interface GotoCompleteAction {
		void doAction();
	}
	
	GotoCompleteAction gotoCompleteAction;
	
	public GotoDialog(Context context, GotoCompleteAction gotoCompleteAction) {
		super(context);
		this.gotoCompleteAction = gotoCompleteAction;
	}

	
	public boolean onKeyDown(int c, KeyEvent event) {
		
		if (c == KeyEvent.KEYCODE_DPAD_CENTER) {
			gotoCompleteAction.doAction();
			this.hide();
		}
		else if (c == KeyEvent.KEYCODE_DPAD_UP) {
			if (current == modName) {
				int i = modNames.indexOf(module.getName());
				if (++i >= modNames.size()) i = 0;
				SWModule newMod = mgr.getModuleByName(modNames.get(i));
				newMod.setKeyText(module.getKeyText());
				module = newMod;
			}
			else if (current == book) {
				module.setKeyText("+book");
			}
			else if (current == chapter) {
				module.setKeyText("+chapter");
			}
			else if (current == verse) {
				module.next();
			}
			fillControls();
		}
		else if (c == KeyEvent.KEYCODE_DPAD_DOWN) {
			if (current == modName) {
				int i = modNames.indexOf(module.getName());
				if (--i < 0) i = modNames.size()-1;
				SWModule newMod = mgr.getModuleByName(modNames.get(i));
				newMod.setKeyText(module.getKeyText());
				module = newMod;
			}
			else if (current == book) {
				module.setKeyText("-book");
			}
			else if (current == chapter) {
				module.setKeyText("-chapter");
			}
			else if (current == verse) {
				module.previous();
			}
			fillControls();
		}
		else if (c == KeyEvent.KEYCODE_DPAD_LEFT) {
			if (current == modName) {
				setCurrent(verse);
			}
			else if (current == book) {
				setCurrent(modName);
			}
			else if (current == chapter) {
				setCurrent(book);
			}
			else if (current == verse) {
				setCurrent(chapter);
			}
		}
		else if (c == KeyEvent.KEYCODE_DPAD_RIGHT) {
			if (current == modName) {
				setCurrent(book);
			}
			else if (current == book) {
				setCurrent(chapter);
			}
			else if (current == chapter) {
				setCurrent(verse);
			}
			else if (current == verse) {
				setCurrent(modName);
			}
		}
		return super.onKeyDown(c, event);
	}
	
	SWModule module;
	TextView modName;
	TextView book;
	TextView chapter;
	TextView verse;
	TextView current;
	Vector<String> modNames = new Vector<String>();
	SWMgr mgr = new SWMgr();
	
	void loadMods() {
		modNames = new Vector<String>();
		
        SWMgr.ModInfo mods[] = mgr.getModInfoList();
        for (SWMgr.ModInfo mi : mods) {
	        SWModule module = mgr.getModuleByName(mi.name);
	        if (module.getCategory().startsWith("Biblica")) {
	        	modNames.add(mi.name);
			}
		}
	}
	
	void askUser(SWModule currentModule) {
		module = currentModule;
		String keyText = module.getKeyText();
		
		// must show before we try to set controls
		show();
		
		this.setTitle(module.getName() + ": " + keyText);
		fillControls();
		setCurrent(book);
	}
	
	void fillControls() {
		modName.setText(module.getName());
		String keyComponents[] = module.getKeyChildren();
		book.setText(keyComponents[6]);
		chapter.setText(keyComponents[2]);
		verse.setText(keyComponents[3]);
	}
	
	void setCurrent(TextView tv) {
		current = tv;
		modName.setBackgroundColor(Color.GRAY);
		book.setBackgroundColor(Color.GRAY);
		chapter.setBackgroundColor(Color.GRAY);
		verse.setBackgroundColor(Color.GRAY);
		
		current.setBackgroundColor(Color.CYAN);
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.setTitle("About The SWORD Project");
		mainLayout = new LinearLayout(this.getContext());
		mainLayout.setOrientation(LinearLayout.HORIZONTAL);
		modName = new TextView(getContext());
		modName.setPadding(10, 10, 10, 10);
		mainLayout.addView(modName);
		book = new TextView(getContext());
		book.setPadding(10, 10, 10, 10);
		mainLayout.addView(book);
		chapter = new TextView(getContext());
		chapter.setPadding(10, 10, 10, 10);
		mainLayout.addView(chapter);
		verse = new TextView(getContext());
		verse.setPadding(10, 10, 10, 10);
		mainLayout.addView(verse);
		addContentView(mainLayout, new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
		loadMods();
	}

}
