package org.crosswire.android.bishop;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Vector;

import org.crosswire.android.sword.InstallMgr;
import org.crosswire.android.sword.SWMgr;
import org.crosswire.android.sword.SWModule;
import org.crosswire.android.sword.SWMgr.ModInfo;

import android.app.ProgressDialog;
import android.app.TabActivity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.ContextMenu;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.Spinner;
import android.widget.TabHost;
import android.widget.TabWidget;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.LinearLayout.LayoutParams;

public class Installer extends TabActivity {
	
	public void onContentChanged() {
		super.onContentChanged();
	}

	private static final int INSTALL_MODULE = Menu.FIRST + 1;
	private static final int ABOUT_INSTALLED = Menu.FIRST + 2;
	private static final int ABOUT_AVAILABLE = Menu.FIRST + 3;
	private static final int INSTALL_VIEW = Menu.FIRST + 4;
	private static final int AVAILABLE_VIEW = Menu.FIRST + 5;
	private static final int REMOVE_MODULE = Menu.FIRST + 6;
	private static final int REPORT_PROGRESS = Menu.FIRST + 7;
	private static final int REFRESH_FINISHED = Menu.FIRST + 8;
	private static final int INSTALL_FINISHED = Menu.FIRST + 9;
	private static final int REFRESH_REMOTE = Menu.FIRST + 10;

    SWMgr mgr = new SWMgr();
	InstallMgr installMgr = new InstallMgr();
    ProgressDialog progress;
    String clickedInstModName;
    String clickedAvailModName;
    About about = null;
	
	ListView availModList;
	Spinner availSources;
	Vector<HashMap<String,String>> sourcesModel = new Vector<HashMap<String,String>>();
	ArrayList<String> sourceList = new ArrayList<String>();
	Vector<HashMap<String,String>> availModulesModel = new Vector<HashMap<String,String>>();
	ArrayAdapter<String> availSourcesAdapter;
	SimpleAdapter availAdapter;
	SimpleAdapter instAdapter; 
	
	ListView instModList;
	Vector<HashMap<String,String>> instModel = new Vector<HashMap<String,String>>();
	

	protected void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);
		
		TabHost tabHost = new TabHost(this);
		tabHost.setId(android.R.id.tabhost);
		
		LinearLayout mainLayout = new LinearLayout(this);
		mainLayout.setOrientation(LinearLayout.VERTICAL);
		tabHost.addView(mainLayout);
		
		TabWidget tabs = new TabWidget(this);
		tabs.setId(android.R.id.tabs);
		mainLayout.addView(tabs);
		
		FrameLayout tabContents = new FrameLayout(this);
		tabContents.setId(android.R.id.tabcontent);
		addInstalledView(tabContents);
		addAvailableView(tabContents);
		mainLayout.addView(tabContents);
		
		setContentView(tabHost, new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
		
		// First tab
		TabHost.TabSpec userTab = tabHost.newTabSpec("Installed");
		userTab.setContent(INSTALL_VIEW);
		userTab.setIndicator("Installed");
		tabHost.addTab(userTab);

		// Second tab
		TabHost.TabSpec secTab = tabHost.newTabSpec("Available");
		secTab.setContent(AVAILABLE_VIEW);
		secTab.setIndicator("Available");
		tabHost.addTab(secTab);
		
	    progress = new ProgressDialog(this);
	    about = new About(this);
	    
	    // TODO: check before release!!!
		installMgr.setUserDisclaimerConfirmed();
		
		String sources[] = installMgr.getRemoteSources();
		if (sources == null || sources.length < 1) {
			updateConfigInfo();
		}
	}
	
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.add(0, REFRESH_REMOTE, 0, "Refresh Remote").setShortcut('1', 'r');
		return true;
	}
	
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case REFRESH_REMOTE:
			updateConfigInfo();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}
	Handler myHandler = new Handler() {

		public void handleMessage(Message msg) {
			switch (msg.what) {
			case REPORT_PROGRESS:
//				progress.setProgress(msg.getData().getInt("progress"));
			    progress.setMessage(msg.getData().getString("message"));
				break;
			case REFRESH_FINISHED:
			case INSTALL_FINISHED:
				progress.setMessage("Finishing up...");
				mgr.reInit();
				installMgr.reInit();
				loadInstalledMods();
				loadSources();
				loadAvailableMods();
				progress.hide();
				break;
			default:
				super.handleMessage(msg);
			}
		}
	};
	
	
	public void updateConfigInfo() {
	    progress.setTitle("Updating Available Resource Knowledge");
//	    progress.setMax(100);
//	    progress.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
	    progress.show();
		new Thread(new Runnable() {
			
			public void run() {
				Message m = myHandler.obtainMessage(REPORT_PROGRESS);
				Bundle b = new Bundle();
				b.putString("message", "Discovering Publishers...");
				m.setData(b);
				myHandler.sendMessage(m);
				
				int result = installMgr.syncConfig();
				if (result != 0) {
					System.err.println("Problem syncing configs. " + result);
				}
				
				for (String source: installMgr.getRemoteSources()) {
					m = myHandler.obtainMessage(REPORT_PROGRESS);
					b = new Bundle();
					b.putString("message", "Sync with " + source + "...");
					m.setData(b);
					myHandler.sendMessage(m);
				    installMgr.refreshRemoteSource(source);
				}
				m = myHandler.obtainMessage(REFRESH_FINISHED);
				myHandler.sendMessage(m);
    		}
		}).start();
	}
	
		
	void addAvailableView(FrameLayout frame) {
		this.setTitle("Available Modules");
		LinearLayout mainLayout = new LinearLayout(this);
		mainLayout.setOrientation(LinearLayout.VERTICAL);
		
		String[] columns = new String[] {"name"};
		int[] names = new int[] {android.R.id.text1};
		availSources = new Spinner(this);
		availSources.setPadding(10, 10, 10, 10);
		
		availSourcesAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, sourceList);       
		availSourcesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);                  
		availSources.setAdapter(availSourcesAdapter);
		availSources.setPrompt("Choose Source");
		availSources.setOnItemSelectedListener(new OnItemSelectedListener() {
			public void onItemSelected(AdapterView<?> arg0, View arg1, int arg2, long arg3) {
				sourceChanged();
			}
			public void onNothingSelected(AdapterView<?> arg0) {
			}
		});

		mainLayout.addView(availSources);
		
		columns = new String[] {"name", "desc"};
		names = new int[] {android.R.id.text1, android.R.id.text2};
		availAdapter = new SimpleAdapter(this, availModulesModel, android.R.layout.simple_list_item_2, columns, names);
		availModList = new ListView(this);
		availModList.setPadding(10, 10, 10, 10);
		availModList.setAdapter(availAdapter);
		availModList.setOnCreateContextMenuListener(this);
		availModList.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
				clickedAvailModName = ((HashMap<String, String>)availModList.getAdapter().getItem(position)).get("name");
			}
		});
		availModList.setOnItemLongClickListener(new OnItemLongClickListener() {
			public boolean onItemLongClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
				clickedAvailModName = ((HashMap<String, String>)availModList.getAdapter().getItem(position)).get("name");
				return false;
			}
		});
		mainLayout.addView(availModList);
		
		frame.addView(mainLayout, new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
		mainLayout.setId(AVAILABLE_VIEW);
		loadSources();
		loadAvailableMods();
	}
		
		
		

		public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
			super.onCreateContextMenu(menu, v, menuInfo);
			
			if (v == availModList) {
				menu.add(0, INSTALL_MODULE, 0, "Install Module").setShortcut('1', 'i');
				menu.add(2, ABOUT_AVAILABLE, 0, "About Module").setShortcut('2', 'a');
			}
			else {
				menu.add(0, REMOVE_MODULE, 0, "Remove Module").setShortcut('1', 'r');
				menu.add(2, ABOUT_INSTALLED, 0, "About Module").setShortcut('2', 'a');
			}
		}


		void loadSources() {
			sourceList.clear();
			for (String source : installMgr.getRemoteSources()) {
				sourceList.add(source);
			}
			availSources.setAdapter(availSourcesAdapter);
		}
		
		void sourceChanged() {
			loadAvailableMods();
		}
		
		
	void loadAvailableMods() {
		availModulesModel.clear();
		String source = (String)availSources.getSelectedItem();
		if (source != null && source.length() > 0) {
			for (ModInfo modInfo : installMgr.getRemoteModInfoList(source)) {
				HashMap<String, String> row = new HashMap<String, String>();
				if (modInfo.category.startsWith("Biblical")) {
					row.put("name", modInfo.name);
					row.put("desc", modInfo.description);
					availModulesModel.add(row);
				}
			}
		}
		availModList.setAdapter(availAdapter);
	}
	
	
	void addInstalledView(FrameLayout frame) {
	
		this.setTitle("Installed Modules");
		LinearLayout mainLayout = new LinearLayout(this);
		mainLayout.setOrientation(LinearLayout.VERTICAL);
		
		String[] columns = new String[] {"name", "desc"};
		int[] names = new int[] {android.R.id.text1, android.R.id.text2};
		instAdapter = new SimpleAdapter(this, instModel, android.R.layout.simple_list_item_2, columns, names);
		instModList = new ListView(this);
		instModList.setPadding(10, 10, 10, 10);
		instModList.setAdapter(instAdapter);
		instModList.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
				clickedInstModName = ((HashMap<String, String>)instModList.getAdapter().getItem(position)).get("name");
			}
		});
		instModList.setOnItemLongClickListener(new OnItemLongClickListener() {
			public boolean onItemLongClick(AdapterView<?> arg0, View arg1, int position, long arg3) {
				clickedInstModName = ((HashMap<String, String>)instModList.getAdapter().getItem(position)).get("name");
				return false;
			}
		});
		instModList.setOnCreateContextMenuListener(this);
		mainLayout.addView(instModList);
		
		frame.addView(mainLayout, new LinearLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));
		mainLayout.setId(INSTALL_VIEW);
		loadInstalledMods();
	}
	
	
	void loadInstalledMods() {
		instModel.clear();
		HashMap<String, String> row;
		for (SWMgr.ModInfo modInfo : mgr.getModInfoList()) {
			row = new HashMap<String, String>();
			row.put("name", modInfo.name);
			row.put("desc", modInfo.description);
			instModel.add(row);
		}
		instModList.setAdapter(instAdapter);
	}
	
	
	
	
	
	
	public boolean onContextItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case INSTALL_MODULE: {
			final String sourceName = (String)availSources.getSelectedItem();
		    progress.setTitle("Installing " + clickedAvailModName);
//	    progress.setMax(100);
//	    progress.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		    progress.show();
			new Thread(new Runnable() {
		
				public void run() {
					Message m = myHandler.obtainMessage(REPORT_PROGRESS);
					Bundle b = new Bundle();
					b.putString("message", "Downloading...");
					m.setData(b);
					myHandler.sendMessage(m);
			
					int result = installMgr.remoteInstallModule(sourceName, clickedAvailModName);
					if (result != 0) {
						System.err.println("Problem installing: " + result);
					}
			
					m = myHandler.obtainMessage(INSTALL_FINISHED);
					myHandler.sendMessage(m);
	    		}
			}).start();
			return true;
		}
		
		case ABOUT_AVAILABLE: {
			String sourceName = (String)availSources.getSelectedItem();
			SWModule mod = installMgr.getRemoteModuleByName(sourceName, clickedAvailModName);
			about.setPurpose(mod);
			about.show();
			return true;
		}
		case ABOUT_INSTALLED: {
			SWModule mod = mgr.getModuleByName(clickedInstModName);
			about.setPurpose(mod);
			about.show();
			return true;
		}
		
		case REMOVE_MODULE:
			new InstallMgr().uninstallModule(clickedInstModName);
			mgr.reInit();
			loadInstalledMods();
			return true;
		}
		return super.onContextItemSelected(item);
	}

}
